import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

class CompanyUsingBinFiles {

  final static char FIELD_TERMINATOR = ',';

  // Field
  private PersonnelRegister register;

  // Constructor
  CompanyUsingBinFiles() {
    register = new PersonnelRegister();
  }
  CompanyUsingBinFiles(PersonnelRegister register) {
    this.register = register;
  }

  // Print statistics
  void printReport() {
    System.out.println(register);
  }
  void printAllEmployeesToTerminalWindow() {
    Employee[] employees = register.getEmployeeArray();
    int numOfEmployees = register.getNumOfEmployees();
    System.out.println("Number of employees: " + numOfEmployees);
    for (int i = 0; i < numOfEmployees; i++) {
      System.out.println(employees[i]);
    }
  }

  void writeAllEmployeesToBinFile(String binaryFileName, boolean append) {
    try {
      FileOutputStream outFileStream
                 = new FileOutputStream(binaryFileName, append);  // (4)
      DataOutputStream outBinStream
                 = new DataOutputStream(outFileStream);           // (5)

      // Write the number of employees in the register.
      int numOfEmployees = register.getNumOfEmployees();
      outBinStream.writeInt(numOfEmployees);                      // (6a)

      // Write data about each employee.
      Employee[] employees = register.getEmployeeArray();
      for (int i = 0; i < numOfEmployees; i++)
        writeEmployeeData(outBinStream, employees[i]);            // (6b)

      outBinStream.close();                                       // (7)
    }
    catch (FileNotFoundException exception) {                     // (8)
      System.out.print("Error opening file: " + binaryFileName);
      System.exit(1); // Terminate if an exception occurs.
    }
    catch (IOException exception) {                               // (9)
      System.out.print("Error writing to file: " + exception);
      System.exit(1); // Terminate if an exception occurs.
    }
  }

  void writeEmployeeData(DataOutputStream outBinStream,
                         Employee employee) throws IOException {  // (10)
    outBinStream.writeChars(employee.firstName + FIELD_TERMINATOR);
    outBinStream.writeChars(employee.lastName  + FIELD_TERMINATOR);
    outBinStream.writeDouble(employee.hourlyRate);
    outBinStream.writeInt(employee.gender.ordinal());
  }

  void readAllEmployeesFromBinFile(String binaryFileName) {
    try {
      FileInputStream inFileStream
                      = new FileInputStream(binaryFileName);      // (11)
      DataInputStream inBinStream
                      = new DataInputStream(inFileStream);        // (12)

      // Read how many employees are in the binary file.
      int numOfEmployees = inBinStream.readInt();                 // (13)

      // Create a personnel register.
      register = new PersonnelRegister();

      // Read data about all employees in the file.
      for (int i = 0; i < numOfEmployees; i++) {
        // Read an employee.
        Employee employee = readEmployeeData(inBinStream);        // (14)
        // Register the employee.
        register.registerEmployee(employee);
      }

      inBinStream.close();                                        // (15)
    }
    catch (IOException exception) {
      System.out.print("Error writing to file: " + exception);
      System.exit(1); // Terminate if exception occurs.
    }
  }

  Employee readEmployeeData(DataInputStream inBinStream)
                            throws IOException {                  // (16)
    String firstName   = readBinStringField(inBinStream);
    String lastName    = readBinStringField(inBinStream);
    double hourlyRate  = inBinStream.readDouble();
    Gender gender = null;
    switch (inBinStream.readInt()) { // Read ordinal value for gender.
      case 0: default:
        gender = Gender.FEMALE;
        break;
      case 1:
        gender = Gender.MALE;
        break;
    }
    return new Employee(firstName, lastName, hourlyRate, gender);
  }

  private String readBinStringField(DataInputStream inBinStream)  // (17)
                 throws IOException {
    // Assumes that the field is terminated by FIELD_TERMINATOR.
    StringBuilder charBuffer = new StringBuilder();
    while(true) {
      char character = inBinStream.readChar();
      if (character == FIELD_TERMINATOR)
        break;
      else
        charBuffer.append(character);
    }
    return charBuffer.toString();
  }
}