import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

class CompanyUsingTextFiles {

  final static char FIELD_TERMINATOR = ',';  // Comma

  // Field
  private PersonnelRegister register;

  // Constructors
  CompanyUsingTextFiles() {
    register = new PersonnelRegister();
  }

  CompanyUsingTextFiles(PersonnelRegister register) {
    this.register = register;
  }

  // Print statistics
  void printReport() {
    System.out.println(register);
  }

  void printAllEmployeesToTerminalWindow() {
    Employee[] employees = register.getEmployeeArray();
    int numOfEmployees = register.getNumOfEmployees();
    System.out.println("Number of employees: " + numOfEmployees);
    for (int i = 0; i < numOfEmployees; i++) {
      System.out.println(employees[i]);
    }
  }

  void writeAllEmployeesToTextFile(String dataFileName)
       throws IOException {                                        // (4)

    FileWriter textFileWriter = new FileWriter(dataFileName);      // (5)
    PrintWriter textWriter = new PrintWriter(textFileWriter);      // (6)

    // Write the number of employees in the register.
    int numOfEmployees = register.getNumOfEmployees();
    textWriter.println(numOfEmployees);                            // (7)

    // Write data about each employee.
    Employee[] employees = register.getEmployeeArray();
    for (int i = 0; i < numOfEmployees; i++) {
      writeEmployeeData(textWriter, employees[i]);                 // (8)
    }

    textWriter.close();                                            // (9)
  }

  void writeEmployeeData(PrintWriter textWriter,
                         Employee employee) {                      // (10)
    // Fields separated by a field terminator character.
    textWriter.printf(
        "%s" + FIELD_TERMINATOR + "%s" + FIELD_TERMINATOR +
        "%.2f" + FIELD_TERMINATOR + "%s%n",
        employee.firstName, employee.lastName,
        employee.hourlyRate, employee.gender);
  }

  void readAllEmployeesFromTextFile(String dataFileName)
       throws IOException {                                        // (11)

    FileReader textFileReader = new FileReader(dataFileName);      // (12)
    BufferedReader textReader = new BufferedReader(textFileReader);// (13)

    // Read how many employees are in the text file.
    String firstLine = textReader.readLine();                      // (14)
    int numOfEmployees = Integer.parseInt(firstLine);

    // Create a personnel register.
    register = new PersonnelRegister();

    // Read data about all employees in the file.
    for (int i = 0; i < numOfEmployees; i++) {
      // Read an employee.
      Employee employee = readEmployeeData(textReader);            // (15)
      // Register the employee.
      register.registerEmployee(employee);
    }

    textReader.close();                                            // (16)
  }

  Employee readEmployeeData(BufferedReader textReader)
           throws IOException {                                    // (17)
    // Read a record, i.e. a text line.
    String record = textReader.readLine();                         // (18)

    // Find the index of the field terminator characters in the record.
    int fieldTerminatorIndex1 = record.indexOf(FIELD_TERMINATOR);  // (19)
    int fieldTerminatorIndex2 = record.indexOf(FIELD_TERMINATOR,
                                          fieldTerminatorIndex1 + 1);
    int fieldTerminatorIndex3 = record.indexOf(FIELD_TERMINATOR,
                                          fieldTerminatorIndex2 + 1);

    // Extract the values of the fields from the data record.
    String firstName = record.substring(0, fieldTerminatorIndex1); // (20)
    String lastName = record.substring(fieldTerminatorIndex1 + 1,
                                       fieldTerminatorIndex2);

    String doubleStr = record.substring(fieldTerminatorIndex2 + 1,
                                        fieldTerminatorIndex3);
    double hourlyRate  = Double.parseDouble(doubleStr);            // (21)

    String genderStr = record.substring(fieldTerminatorIndex3 + 1);
    Gender gender;
    if (genderStr.equals(Gender.MALE.toString())) {                // (22)
      gender = Gender.MALE;
    } else {
      gender = Gender.FEMALE;
    }

    return new Employee(firstName, lastName, hourlyRate, gender);
  }
}